import { defineStore } from 'pinia'

import { publicAxios } from '@/services/authService'
import { getPaginatedResult } from '@/stores/helpers'
import {
  deleteFile,
  deleteItem,
  deleteItems,
  deleteItemThumbnail,
  getItem,
  getSetItems,
  patchItem,
  postItem,
  postItemCsvImport,
  postItemFile, postItemRti,
  postItemThumbnail,
  postUrl,
} from '@/services/itemsService'
import { useSetStore } from './set'
import { notifyError, notifySuccess } from '@/services/alertService'
import { t } from '@/plugins/i18n'

import type { DuplicationTarget, ItemStoreState, NewItem, PountItem, SetItem } from '#/item'
import type { MediaFile, RendererName } from '#/file'

/**
 * @typedef {Object} SetItem
 * @property {boolean} canEdit - rights for project manager
 * @property {string} description
 * @property {string} id
 * @property {boolean} isPublic
 * @property {ThumbnailSource} thumbnails
 * @property {string} title
 * @property {string} updatedAt
 */

/**
 * @typedef {Object} MediaFile
 * @property {string} id
 * @property {string} contentType
 * @property {Object|null} [info] - size & content type
 * @property {string} name
 * @property {string} size
 * @property {string} updatedAt
 * @property {string} url - s3 url
 */

/**
 * @typedef {string|string[]|Array<string[]>} ItemValue
 * type depends on Item multiple and repeatable properties
 */

/**
 * @typedef {Object} Metadata
 * @property {string} id - template field id
 * @property {string} name - template field name
 * @property {ItemValue} value - field value, values or list of values
 * @property {boolean} multiple - template field allow multiple values
 * @property {boolean} isOption - its value is an option of template field
 *                                see MetaSaveMixin
 * @property {boolean} repeatable - values will be dispatch into duplicated fields
 */

/**
 * @typedef {Object} FileViewer
 * @property {string} [component] - name of component used to read the file
 * @property {string} [fileId] - id of the MediaFile to be read
 * @property {boolean} isDownloadable - control download button visibility
 */

/**
 * @typedef {object} ThumbnailSource
 * @property {string|null} s - url to object's 48×48 thumbnail
 * @property {string|null} m - url to object's 64×64 thumbnail
 * @property {string|null} l - url to object's 128×128 thumbnail
 * @property {string|null} embed - url to object's 512×512 thumbnail
 */

/**
 * @typedef {Object} ItemCreator
 * @property {string} id
 * @property {string} name
 * @property {string} username
 * @property {string} email
 */

/**
 * @typedef {Object} PountItem
 * @property {boolean} canEdit - true if user is project manager or creator of this item
 * @property {boolean} canBeShared - true if itself, its parents set and project are public
 * @property {ItemCreator | null} creator
 * @property {string | null} description
 * @property {string} [id]
 * @property {boolean} isPublic
 * @property {MediaFile[]} mediaFiles
 * @property {Metadata[]} metadata -
 * @property {Object} set - parent set details
 * @property {ThumbnailSource} thumbnails - urls of thumbnails
 * @property {string} title - 255 chars max
 * @property {FileViewer | null} viewer - viewer parameters
 */


const initItemStore = (): ItemStoreState => ({
  current: initItem(),
  setItems: [],
  setItemsCount: 0,
  pageCount: 1
})

const initItem = (): NewItem | PountItem => ({
  canEdit: false,
  canBeShared: false,
  creator: {
    email: '',
    id: '',
    name: '',
    username: '',
  },
  description: '',
  id: '',
  isPublic: false,
  metadata: [],
  set: {},
  thumbnails: {
    s: undefined,
    m: undefined,
    l: undefined
  },
  title: '',
  viewer: null
})

const SUPPORTED_3D_EXTENSIONS = ['ply', 'nxz', 'nxs']

const TYPE_RENDER_MAP: Record<string, {component: RendererName, icon: string}> = {
  audio: {
    component: 'Audio',
    icon: 'mdi-file-music'
  },
  image: {
    component: 'Image',
    icon: 'mdi-file-image'
  },
  model: {
    component: 'Model',
    icon: 'mdi-video-3d'
  },
  pdf: {
    component: 'Pdf',
    icon: 'mdi-file-pdf-outline'
  },
  rti: {
    component: 'Rti',
    icon: 'mdi-camera-iris'
  },
  video: {
    component: 'Video',
    icon: 'mdi-camera-iris'
  },
}

export { SUPPORTED_3D_EXTENSIONS, TYPE_RENDER_MAP, initItem }

export const useItemStore = defineStore('Item', {
  state: () => initItemStore(),
  getters: {
    itemId: (state) => (state.current as PountItem).id,
    set: (state) => (state.current as PountItem).set,
    templateId: (state) => (state.current as PountItem).set?.templateId,
    projectId: (state) => (state.current as PountItem).set?.projectId,
    canEdit: (state): boolean => Boolean((state.current as PountItem).canEdit),
    metadata: (state) => (state.current as PountItem).metadata,
    files: (state) => (state.current as PountItem).mediaFiles,
    fileView: (state): MediaFile | null => {
      const {viewer, mediaFiles} = (state.current as PountItem)
      return (viewer && mediaFiles && viewer.fileId )
          ? <MediaFile>mediaFiles.find((file) => file.id === viewer.fileId)
          : null
    },
    isDownloadable: (state) => Boolean((state.current as PountItem).viewer?.isDownloadable),
  },
  actions: {
    async fileDelete(fileId: string) {
      const fromItemId = this.itemId
      if (fromItemId) {
        const removeViewer = this.fileView?.id === fileId
        await deleteFile(fromItemId, fileId)
            .then(async () => {
              const item = this.current as PountItem
              item.mediaFiles = item.mediaFiles.filter((file) => file.id !== fileId)
              if (removeViewer) {
                await this.itemUpdate({ viewer: null })
              }
            })
      }
    },
    addMediaFile(mediaFile: MediaFile) {
      (this.current as PountItem).mediaFiles.unshift(mediaFile)
    },
    /**
     *
     * @param {File} file
     * @returns {Promise<*>}
     */
    fileUpload(file: File) {
      const itemId = this.itemId as string
      const { name: filename, type: contentType} = file
      // request pre-signed post url
      return postUrl(itemId, filename, contentType)
        .then((response) => {
          const {url, fields} = response as {url: string, fields: Record<string, string>}
          const formData = new FormData()
          formData.append("Content-Type", contentType)
          Object.entries(fields).forEach(([k, v]) => {
            formData.append(k, v)
          });
          formData.append("file", file)
          // post file to s3 storage
          return publicAxios.post(url, formData)
            .then(() => {
              // link file to item (mediaFile creation)
              return postItemFile(itemId, filename)
                .then((mediaFile: MediaFile) => {
                  this.addMediaFile(mediaFile)
                  notifySuccess(t('file.upload.success', { filename }))
                  return 100
                })
            })
        })
        .catch((error) => {
          notifyError(t('file.upload.error', {filename}), error)
          return 0
      })
    },
    rtiUpload(file: File) {
      const filename = file.name
      const itemId = this.itemId as string
      const formData = new FormData()
      formData.append("Content-Type", file.type)
      formData.append("file", file)
      return postItemRti(itemId, formData)
        .then((mediaFile: MediaFile) => {
          this.addMediaFile(mediaFile)
          notifySuccess(t('file.upload.success', { filename }))
          return 100
        }).catch(error => {
          notifyError(t('file.upload.error', {filename}), error)
          return 0
        })
    },
    setCurrentItem(item: PountItem | NewItem) {
      this.current = { ...item}
    },
    /** Create an Item for current project and load it into current item. */
    itemCreate(item: NewItem) {
      const setStore = useSetStore()
      const { setId } = setStore
      const itemData = {...item, setId}

      return postItem(itemData)
        .then(this.setCurrentItem)
        .catch((error) => {
          notifyError(t('notify.error.item.create'), error)
        })
    },
    itemDelete(itemId: string) {
      return deleteItem(itemId)
        .then(() => {
          this.setCurrentItem(initItem())
          notifySuccess(t('notify.delete.success'))
        })
        .catch((error) => {
          notifyError(t('notify.error.item.delete', 1), error)
        })
    },
    async itemCSVImport(file: File) {
      const setStore = useSetStore()
      const { setId } = setStore
      const formData = new FormData()
      formData.append("Content-Type", file.type)
      formData.append("csv", file)
      formData.append("set_id", setId as string)
      return await postItemCsvImport(formData)
    },
    /**
     * Create a copy of current item in the same set
     * changing its title and resetting associated files
     */
    itemDuplicate(target?: DuplicationTarget) {
      const item = (this.current as PountItem)
      const {title: oldTitle, description, isPublic} = item
      const title = `${t('item.copy.prefix')}${oldTitle}`

      const metadata = (target?.metadata
          ? target.metadata
          : item.metadata)
          .map(d => d.name === 'title'
              ? { ...d, value: title}
              : d
          )
      const setId = target?.setId ? target.setId : item.set.id
      if (setId === undefined) {
       throw new TypeError('setId is mandatory, see no orphan item policy')
      }

      const itemData = {
        description,
        isPublic,
        metadata,
        setId,
        title,
      }
      return postItem(itemData)
        .then((fetchedItem) => {
          this.setCurrentItem(fetchedItem)
          this.router.push({ name: 'itemViewer', params: { itemId: fetchedItem.id } })
        })
        .catch((error) => {
          notifyError(t('notify.error.item.create'), error)
        })
    },
    /** delete a list of items */
    itemMultiDelete(itemIds: string[]) {
      return deleteItems(itemIds)
          .then(() => {
            this.setItems = this.setItems.filter((item: SetItem) => !itemIds.includes(item.id))
            notifySuccess(t('notify.delete.success'))
          })
          .catch((error) => {
            notifyError(t('notify.error.item.delete', itemIds.length), error)
          })
    },
    /** Retrieve from api, item of given id and load it into current item. */
    itemLoad(itemId: string) {
      return getItem(itemId)
          .then((item) => {
            this.setCurrentItem(item)
          })
          .catch((error) => {
            notifyError(t('notify.error.item.load'), error)
          })
    },
    /** Update current item fields, one or many among title, description and publisher */
    itemUpdate(itemPatch: Partial<PountItem>) {
      const itemId = (this.current as PountItem).id as string
      return patchItem(itemId, itemPatch)
          .then((item) => {
            this.setCurrentItem(item)
            notifySuccess(t('item.updated'))
          })
          .catch((error) => {
            notifyError(t('notify.error.update'), error)
          })
    },
    setItemsLoad(page = 1, size = 0) {
      const setStore = useSetStore()
      const setId = setStore.setId
      return getSetItems(setId, page, size)
          .then((paginatedItems) => {
            const {pageContent, pageCount, total} = getPaginatedResult<SetItem>(paginatedItems, page, size)
            this.setItems = pageContent
            this.setItemsCount = total
            this.pageCount = pageCount
            return total
          })
          .catch((error) => {
            notifyError(t('notify.error.item.all'), error)
          })
    },
    async thumbnailUpdate(objectId: string, file: File) {
      const { name: filename } = file
      const formData = new FormData()
      formData.append("Content-Type", file.type)
      formData.append("file", file)
      await postItemThumbnail(objectId, formData)
          .then((thumbnails) => {
            (this.current as PountItem).thumbnails = thumbnails
            notifySuccess(t('file.upload.success', {filename}))
          })
          .catch((error) => {
            notifyError(t('file.upload.error', {filename}), error)
          })
    },
    thumbnailDelete(objectId: string) {
      return deleteItemThumbnail(objectId)
        .then(() => {
          (this.current as PountItem).thumbnails = {};
          notifySuccess(t('file.delete.success'));
        })
        .catch((error) => {
          notifyError(t('file.delete.error'), error)
        })
    },
  },
})
