import { describe, it, expect, beforeEach, afterEach, vi } from 'vitest'
import { createPinia, setActivePinia } from 'pinia'

import { publicAxios, authAxios } from '@/services/authService'
import { t } from '@/plugins/i18n'
import { notifyError, notifySuccess } from '@/services/alertService'
import { useItemStore, initItem } from '@/stores/item'
import { useSetStore } from '@/stores/set'

import type { Mocked, MockInstance } from 'vitest'
import type { Authority } from '@/services/authService'
import type { Axios } from 'axios'
import type { PountItem, Metadata } from '#/item'
import type { MediaFile } from '#/file'
import type { DuplicationTarget } from '../../../types/item'


vi.mock(
    '@/services/alertService',
    () => ({
      notifyError: vi.fn(),
      notifySuccess: vi.fn()
    })
)

vi.mock(
    '@/services/authService',
    () => ({
      authAxios: {
        delete: vi.fn(),
        get: vi.fn(),
        patch: vi.fn(),
        post: vi.fn(),
      },
      publicAxios: {
        get: vi.fn(),
        patch: vi.fn(),
        post: vi.fn(),
      }
    })
)

const mockedAxios: Mocked<Authority> = vi.mocked(authAxios, true)
const mockedPublicAxios: Mocked<Axios> = vi.mocked(publicAxios, true)

const creator = {
  id: '123aa-a653c',
  name: 'The Creator',
  username: 'matrix.0',
  email: 'creator@www.org'
}
const metadata = [
  {id: 'x', name: 'title', value: 'previous title',
    multiple: false, isOption: false, repeatable: false},
  {id: 'y', name: 'creator', value: ['yourself'],
    multiple: false, isOption: false, repeatable: true},
]

const set = {
  id: '2345e-ec423',
  title: 'Item container',
  templateId: 'template-uuid',
}

const thumbnails = {
  s: 'some.url.s',
  m: 'some.url.m',
  l: 'some.url.l',
  embed: 'some.url.embed'
}

const file = {
  id: '54534',
  name: 'picture.jpg',
  contentType: 'image/jpg',
  info: {contentType: 'image/jpg', size: 10240},
  size: '12k',
  updatedAt: '2021-06-03T16:17:11.268946+02:00',
  url: 'some.url',
}

describe('store/item', () => {

  setActivePinia(createPinia())
  const store = useItemStore()
  const setStore = useSetStore()

  describe('actions', () => {
    const axiosError = new Error('failed')
    const testError = (apiMethod: 'get' | 'patch' | 'post' | 'delete', message: string, storeAction: Function, ...params: any[]) =>
        async () => {
          mockedAxios[apiMethod].mockRejectedValueOnce(axiosError)
          await storeAction(...params)
          expect(notifyError).toHaveBeenCalledOnce()
          expect(notifyError).toHaveBeenLastCalledWith(
              message,
              axiosError
          )
        }
    const testSuccess = (message: string) => {
      expect(notifySuccess).toHaveBeenNthCalledWith(1, message)
    }

    const setId = '48e9-6a97'
    beforeEach(() => {
      store.$reset()
      setStore.current.id = setId
    })
    afterEach(() => {
      vi.restoreAllMocks()
    })
    describe('itemCreate', () => {
      const newItem = {
        title: 'item title',
        description: 'au moins un mot',
        isPublic: false,
        metadata: [],
        thumbnails: {},
      }
      it('should post item and setId to api', () => {
        store.itemCreate(newItem)
        expect(mockedAxios.post).toHaveBeenCalledOnce()
        expect(mockedAxios.post).toHaveBeenLastCalledWith(
            'items/',
            {setId, ...newItem}
        )
      })
      it('should update current item with retrieve data', async () => {
        const retrieveItem = {
          id: '32d3-5478',
          title: 'item title',
          description: 'au moins un mot',
          isPublic: false,
          canEdit: true,
          metadata: [],
          creator: {},
          set: {id: setId},
          thumbnails: {},
        }
        mockedAxios.post.mockResolvedValueOnce({data: retrieveItem})
        await store.itemCreate(newItem)
        expect(store.current).toEqual(retrieveItem)
      })
      it('should notify explicit error', testError(
          'post',
          t('notify.error.item.create'),
          store.itemCreate, newItem
      ))
    })
    describe('itemCSVImport', () => {
      let actionResponse: {createdItems: number}
      const csvFile = new File(
          ["header1,header2\nvalue1,value2"],
          "items.csv",
          {type: "text/csv"}
      )
      beforeEach(async () => {
        mockedAxios.post.mockResolvedValueOnce({data: {createdItems: 5}})
        actionResponse = await store.itemCSVImport(csvFile)
      })
      it('should post to specific item api action', () => {
        expect(mockedAxios.post).toHaveBeenCalledOnce()
        expect((mockedAxios.post.mock.lastCall as any[])[0]).toEqual('items/import_csv/')
      })
      it('should post specific formData object', () => {
        const postPayload = (mockedAxios.post.mock.lastCall as any[])[1]
        expect(postPayload).to.be.instanceOf(FormData, 'payload class')
        expect(postPayload.get('Content-Type')).toEqual("text/csv")
        expect(postPayload.get('csv')).toEqual(csvFile)
        expect(postPayload.get('set_id')).toEqual(setId)
      })
      it('should return created items count from response', () => {
        expect(actionResponse).toEqual({createdItems: 5})
      })
    })
    describe('itemDuplicate', () => {
      const oldTitle = 'previous title'
      beforeEach(() => {
        store.current = {
          canEdit: false,
          canBeShared: false,
          creator,
          description: 'previous description',
          id: '1536',
          isPublic: true,
          mediaFiles: [file],
          metadata,
          set,
          thumbnails,
          title: oldTitle,
          viewer: { isDownloadable: false, fileId: '54534', component: "Image" }
        }
      })
      it('should post an item to api', async () => {
        await store.itemDuplicate()
        expect(mockedAxios.post).toHaveBeenCalledOnce()
        expect((mockedAxios.post.mock.lastCall as any[])[0]).toEqual('items/')
      })
      it('should update current item with retrieve data', async () => {
        const retrieveItem = {
          id: '32d3-5478',
          title: 'item title',
          description: 'au moins un mot',
          isPublic: false,
          canEdit: true,
          metadata: {},
          creator: {},
          set: {id: setId}
        }
        mockedAxios.post.mockResolvedValueOnce({data: retrieveItem})
        await store.itemDuplicate()
        expect(store.current).toEqual(retrieveItem)
      })
      it('should notify explicit error', testError(
          'post',
          t('notify.error.item.create'),
          store.itemDuplicate
      ))
      describe('item creation payload', () => {
        describe('without given target', () => {
          let payload: PountItem, title: string
          beforeEach(async () => {
            store.itemDuplicate()
            payload = <PountItem>(mockedAxios.post.mock.lastCall as any[])[1]
            title = <string>payload.title
          })
          it('should have consistant title values', () => {
            const metaTitle = (payload.metadata.find(d => d.name === 'title') as Metadata).value
            expect(title === metaTitle, 'same titles').to.be.true
          })
          it('should have prefixed titles', () => {
            expect(title.startsWith(t('item.copy.prefix'))).to.be.true
          })
          it('should refer to old title', () => {
            expect(title.endsWith(oldTitle), 'title prefix').to.be.true
          })
          it('should let other metadata unchanged', () => {
            expect(payload.metadata.filter(d => d.name !== 'title'))
                .toEqual(metadata.filter(d => d.name !== 'title'))
          })
          it('should belong to the same set', () => {
            expect(payload.setId, 'same id').toEqual(set.id)
          })
        })
        describe('with given target', () => {
          const duplicateInTarget = (target: DuplicationTarget) => () => store.itemDuplicate(target)
          const partialMetadata = [{
            id: '5646-987',
            name: 'title',
            value: 'overridden'
          }]
          const launchCopy = duplicateInTarget({
            metadata: partialMetadata,
            setId: 'targetedSet'
          })
          it('should have consistant title values', async () => {
            await launchCopy()
            const { title, metadata } = (mockedAxios.post.mock.lastCall as any[])[1]
            const metaTitle = metadata.find((d: Metadata) => d.name === 'title').value

            expect(title === metaTitle, 'same titles').to.be.true
            expect(title, 'item.title overrode').to.not.contain('overridden')
          })
          it('should post only given metadata', async () => {
            await launchCopy()
            const { metadata } = (mockedAxios.post.mock.lastCall as any[])[1]
            expect(metadata).toHaveLength(1)
            expect(Object.keys(metadata[0])).toHaveLength(3)
          })
          it('should should create item in given set', async () => {
            await launchCopy()
            const { setId } = (mockedAxios.post.mock.lastCall as any[])[1]
            expect(setId).toEqual('targetedSet')
          })
        })
      })
    })
    describe('itemLoad', () => {
      const retrieveItem = {
        id: '32d3-5478',
        title: 'item title',
        description: 'au moins un mot',
        isPublic: false,
        canEdit: true,
        metadata: {},
        creator: {},
        set: {id: setId}
      }
      describe('success scenario', () => {
        beforeEach(async () => {
          mockedAxios.get.mockResolvedValueOnce({data: retrieveItem})
          await store.itemLoad(retrieveItem.id)
        })
        it('should call item detail api', async () => {
          expect(mockedAxios.get).toHaveBeenNthCalledWith(1, `items/${retrieveItem.id}/`)
        })
        it('should update current item with retrieve data', async () => {
          expect(store.current).toEqual(retrieveItem)
        })
      })
      it('should notify explicit error', testError(
          'get',
          t('notify.error.item.load'),
          store.itemLoad, retrieveItem.id
      ))
    })
    describe('itemUpdate', () => {
      describe('success scenario', () => {
        const generateStoredItem = () => ({
          id: '32d3-5478',
          title: 'item title',
          description: 'au moins un mot',
          isPublic: false,
          canEdit: true,
          metadata: [],
          set: {id: setId},
          thumbnails: {},
        })
        const storedItem = generateStoredItem()
        const patchData = {title: 'new title'}
        const retrieveItem = {
          ...storedItem,
          ...patchData,
        }
        beforeEach(async () => {
          store.current = generateStoredItem()
          mockedAxios.patch.mockResolvedValueOnce({data: retrieveItem})
          await store.itemUpdate(patchData)
        })
        it('should patch item to api', () => {
          expect(mockedAxios.patch).toHaveBeenCalledOnce()
          expect(mockedAxios.patch).toHaveBeenLastCalledWith(
              `items/${storedItem.id}/`,
              patchData,
          )
        })
        it('should update current item with retrieve data', async () => {
          expect(store.current).toEqual(retrieveItem)
        })
        it('should notify it succeed', () => {
          testSuccess(t('item.updated'))
        })
      })
      it('should notify explicit error', testError(
          'patch',
          t('notify.error.update'),
          store.itemUpdate
      ))
    })
    describe('deletion', () => {
      const rubbishItem = {
        canEdit: true,
        description: 'au moins un mot',
        id: '32d3-5478',
        isPublic: true,
        metadata: [],
        thumbnails: {},
        title: 'item title',
        updatedAt: '2021-12-31',
      }
      describe('itemDelete', () => {
        describe('success scenario', () => {
          let setCurrentSpy: MockInstance
          beforeEach(async () => {
            mockedAxios.delete.mockResolvedValueOnce({})
            store.current = rubbishItem
            setCurrentSpy = vi.spyOn(store, 'setCurrentItem')
            await store.itemDelete(rubbishItem.id)
          })
          it('should call delete api', () => {
            expect(mockedAxios.delete)
                .toHaveBeenNthCalledWith(1, `items/${rubbishItem.id}/`)
          })
          it('should reset current item', () => {
            expect(setCurrentSpy).toHaveBeenCalledOnce()
            expect(setCurrentSpy).toHaveBeenLastCalledWith(initItem())
          })
          it('should notify that success', async () => {
            testSuccess(t('notify.delete.success'))
          })
        })
        it('should notify explicit error', testError(
            'delete',
            t('notify.error.item.delete', 1),
            store.itemDelete, ''
        ))
      })
      describe('itemMultiDelete', () => {
        describe('success scenario', () => {
          const anotherItem = {
            canEdit: true,
            description: 'au - un mot',
            id: '687a-865e',
            isPublic: true,
            metadata: [],
            thumbnails: {},
            title: 'another title',
            updatedAt: '2022-02-22',
          }
          beforeEach(async () => {
            mockedAxios.delete.mockResolvedValueOnce({data: { ...rubbishItem, isPublic: false }})
            store.setItems = [rubbishItem, anotherItem]
            await store.itemMultiDelete([rubbishItem.id])
          })
          it('should call delete api', () => {
            expect(mockedAxios.delete).toHaveBeenCalledOnce()
            expect(mockedAxios.delete).toHaveBeenLastCalledWith(
                'items/',
                {data: {itemIds: [rubbishItem.id]}}
            )
          })
          it('should remove list from set items', () => {
            expect(store.setItems, 'one set removed').toHaveLength(1)
          })
          it('should notify that success', () => {
            testSuccess(t('notify.delete.success'))
          })
        })
        it('should notify explicit error', testError(
            'delete',
            t('notify.error.item.delete', 2),
            store.itemMultiDelete, ['', '']
        ))
      })
    })
    describe('fileDelete', () => {
      const itemId = '436a-6c03'
      const viewFileId = '32b9'
      const fileId = '2e2a'
      let itemUpdateSpy: MockInstance
      beforeEach(async () => {
        itemUpdateSpy = vi.spyOn(store, 'itemUpdate')
        store.current = {
          ...initItem(),
          id: itemId,
          mediaFiles: [
            <MediaFile>{id : viewFileId},
            <MediaFile>{id : fileId},
          ],
          viewer: {
            fileId: viewFileId,
            component: 'Audio',
            isDownloadable: true
          },
        }
        await store.fileDelete(viewFileId)
      })
      it('should call delete api', () => {
        expect(mockedAxios.delete).toHaveBeenCalledOnce()
      })
      it('should call forge the proper urls', () => {
        expect(mockedAxios.delete).toHaveBeenLastCalledWith(
            `items/${itemId}/mediafiles/${viewFileId}/`
        )
      })
      it('should remove the file from mediaFile list', () => {
        expect(store.files, 'one file removed').toHaveLength(1)
        expect(store.files[0].id).toEqual(fileId)
      })
      it('should reset viewer if deleted file was in', () => {
        expect(itemUpdateSpy).toHaveBeenCalledOnce()
        expect(itemUpdateSpy).toHaveBeenLastCalledWith({viewer: null})
      })
    })
    describe('fileUpload', () => {
      const itemId = '436a-6c03'
      const file = new File(['content'], 'fileName', {type: 'image/png'})
      const mediaFile = {
        contentType: 'text/plain',
        id: '3674-43a6',
        info: null,
        name: 'filename',
        size: '10o',
        url: 'https://s3',
        updatedAt: '2021-06-03T16:17:11.268946+02:00'
      }
      describe('on expected scenario', () => {
        let addMediaSpy: MockInstance
        beforeEach(async () => {
          mockedPublicAxios.post.mockResolvedValueOnce({})
          store.current = {
            ...initItem(),
            id: itemId,
            mediaFiles: [],
          }
          mockedAxios.post.mockImplementation((url, ) => {
            if (url === `items/${itemId}/post_url/`) {
              return Promise.resolve({ data: {
                  url: 'http://s3post_url.domain.fr',
                  fields: { s3Stuff: 'data' }
                } })
            }
            if (url === `items/${itemId}/mediafiles/`) {
              return Promise.resolve({ data: mediaFile })
            }
            return Promise.resolve()
          })
          addMediaSpy = vi.spyOn(store, 'addMediaFile')
          await store.fileUpload(file)
        })
        it('should first call post_url api', async () => {
          expect(mockedAxios.post).toHaveBeenCalledTimes(2)
          const [url, payload] = mockedAxios.post.mock.calls[0] as [string, {filename: string, contentType: string}]
          expect(url, 'first Call').toEqual(`items/${itemId}/post_url/`)
          expect(payload.filename, 'filename check').toEqual('fileName')
          expect(payload.contentType, 'content type check').toEqual('image/png')
        })
        it('should secondly post file to storage with given url', async () => {
          expect(mockedPublicAxios.post).toHaveBeenCalledOnce()
          const [url, payload] = mockedPublicAxios.post.mock.lastCall as [string, FormData]
          expect(url, 's3 creation api').toEqual('http://s3post_url.domain.fr')
          expect(payload, 'payload check').toBeInstanceOf(FormData)
        })
        it('should post a formData with file and extra fields', async () => {
          const payload = (mockedPublicAxios.post.mock.lastCall as [string, FormData])[1]
          expect(payload.get('Content-Type'), 'content type check').toEqual(file.type)
          expect(payload.get('s3Stuff'), 's3 data added').toEqual('data')
          expect(payload.get('file'), 'file check').toEqual(file)
        })
        it('should thirdly post to create the associated media file', async () => {
          const [url, payload] = mockedAxios.post.mock.lastCall as [string, {filename: string}]
          expect(url, 'third api call').toEqual(`items/${itemId}/mediafiles/`)
          expect(payload, 'payload check').toEqual({filename: file.name})
        })
        it('should thirdly post to create the associated media file', async () => {
          const [url, payload] = mockedAxios.post.mock.lastCall as [string, {filename: string}]
          expect(url, 'third api call').toEqual(`items/${itemId}/mediafiles/`)
          expect(payload, 'payload check').toEqual({filename: file.name})
        })
        it('should add the created media file to current item', () => {
          expect(addMediaSpy).toHaveBeenCalledOnce()
          expect(addMediaSpy).toHaveBeenLastCalledWith(mediaFile)
        })
        it('should notify on success', () => {
          testSuccess(t('file.upload.success', {filename: file.name}))
        })
      })
      describe('with api errors', () => {
        const postUrlResponse = {
          data: {
            url: 'http://s3post_url.domain.fr',
            fields: { s3Stuff: 'data' }
          }
        }
        const notifyFailureTest = async () => {
          await store.fileUpload(file)
          expect(notifyError).toHaveBeenCalledOnce()
          expect(notifyError).toHaveBeenLastCalledWith(
              t('file.upload.error', {filename: 'fileName'}),
              axiosError
          )
        }
        it('should notify error if post url request fail', async () => {
          mockedAxios.post.mockRejectedValueOnce(axiosError)
          await notifyFailureTest()
        })
        it('should notify error if file post fail', async () => {
          mockedPublicAxios.post.mockRejectedValueOnce(axiosError)
          mockedAxios.post.mockResolvedValueOnce(postUrlResponse)
          await notifyFailureTest()
          expect(mockedPublicAxios.post).toHaveBeenCalledOnce()
        })
        it('should notify error if mediaFile creation fail', async () => {
          mockedPublicAxios.post.mockResolvedValueOnce({})
          const actionBackup = store.addMediaFile
          store.addMediaFile = vi.fn()
          mockedAxios.post.mockImplementation((url, ) => {
            if (url.endsWith('post_url/')) {
              return Promise.resolve(postUrlResponse)
            }
            else {
              return Promise.reject(axiosError)
            }
          })
          await notifyFailureTest()
          expect(mockedAxios.post).toHaveBeenCalledTimes(2)
          store.addMediaFile = actionBackup
        })
      })
    })
    describe('addMediaFile', () => {
      const mediaFile = {
        contentType: 'text/plain',
        id: '3674-43a6',
        info: {contentType: 'text/plain', size: 10},
        name: 'filename',
        size: '10o',
        url: 'https://s3',
        updatedAt: '2021-06-03T16:17:11.268946+02:00'
      }
      beforeEach(() => {
        (store.current as PountItem).mediaFiles = []
      })
      it('should add the created media file to current item', async () => {
        await store.addMediaFile(mediaFile)
        expect(store.files, 'media file added').toHaveLength(1)
        expect((store.files as MediaFile[])[0], 'new file').toEqual(mediaFile)
      })
      it('should put new media file at start', async () => {
        const newId = '8794-87e14';
        (store.current as PountItem).mediaFiles.push({...mediaFile, id: newId})
        await store.addMediaFile(mediaFile)
        expect((store.files as MediaFile[])[0].id, 'new file id').toEqual(mediaFile.id)
      })
    })
    describe('rtiUpload', () => {
      let actionResponse: number
      const itemId = '436a-6c03'
      const file = new File(
          ['content'],
          'rti.zip',
          {type: 'application/zip'}
      )
      describe('success scenario', () => {
        const mediaFile = {
          id: '3674-43a6',
          info: null,
        }
        beforeEach(async () => {
          store.current = {
            ...initItem(),
            id: itemId,
            mediaFiles: [],
          }
          mockedAxios.post.mockResolvedValueOnce({ data: mediaFile })
          actionResponse = await store.rtiUpload(file)
        })
        it('should call item api', async () => {
          expect(mockedAxios.post).toHaveBeenCalledOnce()
          expect((mockedAxios.post.mock.lastCall as any[])[0],'url check').toEqual(
              `items/${itemId}/mediafiles/upload_rti/`
          )
        })
        it('should post formData with file', async () => {
          const payload = (mockedAxios.post.mock.lastCall as any[])[1]

          expect(payload, 'payload class').toBeInstanceOf(FormData)
          expect(payload.get('Content-Type')).toEqual("application/zip")
          expect(payload.get('file')).toEqual(file)
        })
        it('should add the created media file to current item', async () => {
          expect(store.files, 'media file added').toHaveLength(1)
          expect((store.files as MediaFile[])[0].id, 'new file id').toEqual('3674-43a6')
        })
        it('should notify on success', async () => {
          testSuccess(t('file.upload.success', {filename: file.name}))
        })
        it('should return 100 on success', () => {
          expect(actionResponse).toEqual(100)
        })
      })
      describe('on failure', () => {
        it('should notify error', testError(
            'post',
            t('file.upload.error', {filename: file.name}),
            store.rtiUpload, file
        ))
        it('should return 0 on failure', async () => {
          mockedAxios.post.mockRejectedValueOnce(axiosError)
          const actionResponse = await store.rtiUpload(file)
          expect(actionResponse).toEqual(0)
        })
      })
    })
    describe('setItemsLoad', () => {
      it('should call retrieve list api', async () => {
        await store.setItemsLoad(1, 11)
        expect(mockedAxios.get).toHaveBeenCalledOnce()
        expect(mockedAxios.get, 'get url check').toHaveBeenLastCalledWith(`items/?set_id=${setId}&page=1&page_size=11`)
      })
      it('should populate set item list with retrieve data', async () => {
        const itemList = [
          {id: 'e321-47bd', title: 'item one'},
          {id: 'e321-47be', title: 'item two'},
          {id: 'e321-47bf', title: 'item four'},
        ]
        mockedAxios.get.mockResolvedValueOnce({
          data: {count: 3, results: itemList}
        })
        await store.setItemsLoad(1, 10)
        expect(store.setItems).toEqual(itemList)
      })
      it('should item pageCount list with retrieve data', async () => {
        mockedAxios.get.mockResolvedValueOnce({
          data: {count: 38, nextPage: 2, results: []}
        })
        await store.setItemsLoad(1, 10)
        expect(store.pageCount).toEqual(4)
      })
      it('should notify on failure', testError(
          'get',
          t('notify.error.item.all'),
          store.setItemsLoad
      ))
    })
  })
})

